@@ -1,7 +1,8 @@
+/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*************************************************************************
  *
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
- * 
+ *
  * Copyright 2000, 2010 Oracle and/or its affiliates.
  *
  * OpenOffice.org - a multi-platform office productivity suite
@@ -52,7 +53,6 @@
 #include <com/sun/star/animations/TransitionSubType.hpp>
 #include <com/sun/star/animations/ValuePair.hpp>
 #include <com/sun/star/animations/AnimationColorSpace.hpp>
-#include <com/sun/star/beans/NamedValue.hpp>
 #include <com/sun/star/drawing/FillStyle.hpp>
 #include <com/sun/star/drawing/LineStyle.hpp>
 #include <com/sun/star/awt/FontWeight.hpp>
@@ -66,9 +66,7 @@
 #include <com/sun/star/container/XChild.hpp>
 #include <comphelper/processfactory.hxx>
 #include <rtl/ustrbuf.hxx>
-#ifndef _RTL_MEMORY_H_
-#include <rtl/memory.hxx>
-#endif
+#include <rtl/memory.h>
 
 #include <vcl/vclenum.hxx>
 #include <svx/svdotext.hxx>
@@ -161,10 +159,10 @@ void ImplTranslateAttribute( rtl::OUString& rString, const TranslateMode eTransl
     }
 }
 
-sal_uInt32 ImplTranslatePresetSubType( const sal_uInt32 nPresetClass, const sal_uInt32 nPresetId, const rtl::OUString& rPresetSubType )
+sal_uInt32 AnimationExporter::TranslatePresetSubType( const sal_uInt32 nPresetClass, const sal_uInt32 nPresetId, const rtl::OUString& rPresetSubType )
 {
-    sal_uInt32	nPresetSubType = 0;
-    sal_Bool	bTranslated = sal_False;
+    sal_uInt32  nPresetSubType = 0;
+    sal_Bool    bTranslated = sal_False;
 
     if ( ( nPresetClass == (sal_uInt32)EffectPresetClass::ENTRANCE ) || ( nPresetClass == (sal_uInt32)EffectPresetClass::EXIT ) )
     {
@@ -241,10 +239,10 @@ sal_uInt32 ImplTranslatePresetSubType( const sal_uInt32 nPresetClass, const sal_
     return nPresetSubType;
 }
 
-const sal_Char* transition::find( const sal_Int16 nType, const sal_Int16 nSubType, const sal_Bool bDirection )
+const sal_Char* AnimationExporter::FindTransitionName( const sal_Int16 nType, const sal_Int16 nSubType, const sal_Bool bDirection )
 {
     const sal_Char* pRet = NULL;
-    int				nFit = 0;
+    int             nFit = 0;
 
     const transition* p = gTransitions;
     while( p->mpName )
@@ -261,7 +259,7 @@ const sal_Char* transition::find( const sal_Int16 nType, const sal_Int16 nSubTyp
             pRet = p->mpName;
             nFit = nF;
         }
-        if ( nFit == 7 )	// maximum
+        if ( nFit == 7 )    // maximum
             break;
         p++;
     }
@@ -283,7 +281,7 @@ SvStream& operator<<(SvStream& rOut, AnimationNode& rNode )
 }
 
 AnimationExporter::AnimationExporter( const EscherSolverContainer& rSolverContainer, ppt::ExSoundCollection& rExSoundCollection ) :
-    mrSolverContainer	( rSolverContainer ), 
+    mrSolverContainer   ( rSolverContainer ),
     mrExSoundCollection ( rExSoundCollection ),
     mnCurrentGroup(0)
 {
@@ -291,7 +289,7 @@ AnimationExporter::AnimationExporter( const EscherSolverContainer& rSolverContai
 
 // --------------------------------------------------------------------
 
-static sal_Int16 GetFillMode( const Reference< XAnimationNode >& xNode, const sal_Int16 nFillDefault )
+sal_Int16 AnimationExporter::GetFillMode( const Reference< XAnimationNode >& xNode, const sal_Int16 nFillDefault )
 {
     sal_Int16 nFill = xNode->getFill();
     if ( ( nFill == AnimationFill::DEFAULT ) ||
@@ -398,7 +396,7 @@ void AnimationExporter::processAfterEffectNodes( const Reference< XAnimationNode
 
                                     while( nLength-- )
                                     {
-                                        if( p->Name.equalsAscii( "master-element" ) )
+                                        if( p->Name.equalsAsciiL( RTL_CONSTASCII_STRINGPARAM( "master-element" ) ) )
                                         {
                                             p->Value >>= xMaster;
                                             break;
@@ -420,7 +418,7 @@ void AnimationExporter::processAfterEffectNodes( const Reference< XAnimationNode
     catch( Exception& e )
     {
         (void)e;
-        DBG_ERROR( "(@CL)AnimationExporter::processAfterEffectNodes(), exception cought!" );
+        OSL_FAIL( "(@CL)AnimationExporter::processAfterEffectNodes(), exception cought!" );
     }
 }
 
@@ -586,14 +584,6 @@ void AnimationExporter::exportNode( SvStream& rStrm, Reference< XAnimationNode >
                 exportAnimPropertySet( rStrm, xNode );
                 exportAnimEvent( rStrm, xNode, 0 );
                 exportAnimValue( rStrm, xNode, sal_False );
-
-    /*
-                EscherExContainer aContainer( rStrm, DFF_msofbtAnimGroup, 1 );
-                exportAnimNode( rStrm, xNode, pParent, nGroupLevel + 1, nFillDefault );
-                exportAnimPropertySet( rStrm, xNode );
-                exportAnimEvent( rStrm, xNode, 0 );
-                exportAnimValue( rStrm, xNode, sal_False );
-    */
             }
             break;
 
@@ -676,7 +666,7 @@ void AnimationExporter::exportNode( SvStream& rStrm, Reference< XAnimationNode >
             }
             break;
 
-            case AnimationNodeType::AUDIO :		// #i58428#
+            case AnimationNodeType::AUDIO :     // #i58428#
             {
                 exportAnimNode( rStrm, xNode, pParent, nGroupLevel, nFillDefault );
                 exportAnimPropertySet( rStrm, xNode );
@@ -777,22 +767,42 @@ Reference< XAnimationNode > AnimationExporter::createAfterEffectNodeClone( const
         Any aEmpty;
         xCloneNode->setBegin( aEmpty );
 
-
         return xCloneNode;
     }
     catch( Exception& e )
     {
         (void)e;
-        DBG_ERROR("(@CL)sd::ppt::AnimationExporter::createAfterEffectNodeClone(), could not create clone!" );
+        OSL_FAIL("(@CL)sd::ppt::AnimationExporter::createAfterEffectNodeClone(), could not create clone!" );
     }
     return xNode;
 }
 
+sal_Bool AnimationExporter::GetNodeType( const Reference< XAnimationNode >& xNode, sal_Int16& nType )
+{
+    // trying to get the nodetype
+    Sequence< NamedValue > aUserData = xNode->getUserData();
+    if ( aUserData.getLength() )
+    {
+    const NamedValue* p = aUserData.getConstArray();
+    sal_Int32 nLength = aUserData.getLength();
+    while( nLength-- )
+    {
+        if( p->Name.equalsAsciiL( RTL_CONSTASCII_STRINGPARAM( "node-type" ) ) )
+        {
+        if ( p->Value >>= nType )
+            return sal_True;
+        }
+    }
+    }
+
+    return sal_False;
+}
+
 void AnimationExporter::exportAnimNode( SvStream& rStrm, const Reference< XAnimationNode >& xNode,
         const ::com::sun::star::uno::Reference< ::com::sun::star::animations::XAnimationNode >*, const sal_Int32, const sal_Int16 nFillDefault )
 {
-    EscherExAtom	aAnimNodeExAtom( rStrm, DFF_msofbtAnimNode );
-    AnimationNode	aAnim;
+    EscherExAtom    aAnimNodeExAtom( rStrm, DFF_msofbtAnimNode );
+    AnimationNode   aAnim;
     rtl_zeroMemory( &aAnim, sizeof( aAnim ) );
 
     // attribute Restart
@@ -805,14 +815,12 @@ void AnimationExporter::exportAnimNode( SvStream& rStrm, const Reference< XAnima
         case AnimationRestart::NEVER : aAnim.mnRestart = 3; break;
     }
 
-    // attribute Fill
-//	aAnim.mnFill = GetFillMode( xNode, pParent );
     switch( nFillDefault )
     {
         default:
         case AnimationFill::DEFAULT : aAnim.mnFill = 0; break;
         case AnimationFill::REMOVE : aAnim.mnFill = 1; break;
-        case AnimationFill::FREEZE : // aAnim.mnFill = 2; break;
+        case AnimationFill::FREEZE :
         case AnimationFill::HOLD :   aAnim.mnFill = 3; break;
         case AnimationFill::TRANSITION : aAnim.mnFill = 4; break;
     }
@@ -836,39 +844,16 @@ void AnimationExporter::exportAnimNode( SvStream& rStrm, const Reference< XAnima
     aAnim.mnGroupType = mso_Anim_GroupType_SEQ;
     switch( xNode->getType() )
     {
-        case AnimationNodeType::PAR :		// PASSTROUGH!!! (as it was intended)
+        case AnimationNodeType::PAR :       // PASSTROUGH!!! (as it was intended)
             aAnim.mnGroupType = mso_Anim_GroupType_PAR;
         case AnimationNodeType::SEQ :
         {
-            // trying to get the nodetype
-            Sequence< NamedValue > aUserData = xNode->getUserData();
-            if ( aUserData.getLength() )
+            sal_Int16 nType = 0;
+            if( GetNodeType( xNode, nType ) )
+            switch( nType )
             {
-                const NamedValue* p = aUserData.getConstArray();
-                sal_Int32 nLength = aUserData.getLength();
-                while( nLength-- )
-                {	
-                    if( p->Name.equalsAsciiL( RTL_CONSTASCII_STRINGPARAM( "node-type" ) ) )
-                    {
-                        sal_Int16 nType = 0;
-                        if ( p->Value >>= nType )
-                        {
-                            switch( nType )
-                            {
-                                case ::com::sun::star::presentation::EffectNodeType::TIMING_ROOT : aAnim.mnNodeType = 0x12; break;
-                                case ::com::sun::star::presentation::EffectNodeType::MAIN_SEQUENCE : aAnim.mnNodeType = 0x18; break;
-        /*
-                                case ::com::sun::star::presentation::EffectNodeType::ON_CLICK :
-                                case ::com::sun::star::presentation::EffectNodeType::WITH_PREVIOUS :
-                                case ::com::sun::star::presentation::EffectNodeType::AFTER_PREVIOUS :
-                                case ::com::sun::star::presentation::EffectNodeType::INTERACTIVE_SEQUENCE :
-                                default:
-        */
-                                }
-                        }
-                        break;
-                    }
-                }
+                case ::com::sun::star::presentation::EffectNodeType::TIMING_ROOT : aAnim.mnNodeType = 0x12; break;
+                case ::com::sun::star::presentation::EffectNodeType::MAIN_SEQUENCE : aAnim.mnNodeType = 0x18; break;
             }
         }
         break;
@@ -901,54 +886,93 @@ void AnimationExporter::exportAnimNode( SvStream& rStrm, const Reference< XAnima
         }
         break;
     }
+
     rStrm << aAnim;
 }
 
+void AnimationExporter::GetUserData( const Sequence< NamedValue >& rUserData, const Any ** pAny, sal_Size nLen )
+{
+    // storing user data into pAny, to allow direct access later
+    rtl_zeroMemory( pAny, nLen );
+    if ( rUserData.getLength() )
+    {
+    const NamedValue* p = rUserData.getConstArray();
+    sal_Int32 nLength = rUserData.getLength();
+    while( nLength-- )
+    {
+        if( p->Name.equalsAsciiL( RTL_CONSTASCII_STRINGPARAM( "node-type" ) ) )
+        {
+        pAny[ DFF_ANIM_NODE_TYPE ] = &(p->Value);
+        }
+        else if ( p->Name.equalsAsciiL( RTL_CONSTASCII_STRINGPARAM( "preset-class" ) ) )
+        {
+        pAny[ DFF_ANIM_PRESET_CLASS ] = &(p->Value);
+        }
+        else if ( p->Name.equalsAsciiL( RTL_CONSTASCII_STRINGPARAM( "preset-id" ) ) )
+        {
+        pAny[ DFF_ANIM_PRESET_ID ] = &(p->Value);
+        }
+        else if ( p->Name.equalsAsciiL( RTL_CONSTASCII_STRINGPARAM( "preset-sub-type" ) ) )
+        {
+        pAny[ DFF_ANIM_PRESET_SUB_TYPE ] = &(p->Value);
+        }
+        else if ( p->Name.equalsAsciiL( RTL_CONSTASCII_STRINGPARAM( "master-element" ) ) )
+        {
+        pAny[ DFF_ANIM_AFTEREFFECT ] = &(p->Value);;
+        }
+        p++;
+    }
+    }
+}
+
+sal_uInt32 AnimationExporter::GetPresetID( const rtl::OUString& rPreset, sal_uInt32 nAPIPresetClass, sal_Bool& bPresetId )
+{
+    sal_uInt32 nPresetId = 0;
+    bPresetId = sal_False;
+
+    if ( rPreset.match( rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( "ppt_" ) ), 0 ) )
+    {
+    sal_Int32 nLast = rPreset.lastIndexOf( '_' );
+    if ( ( nLast != -1 ) && ( ( nLast + 1 ) < rPreset.getLength() ) )
+    {
+        rtl::OUString aNumber( rPreset.copy( nLast + 1 ) );
+        nPresetId = aNumber.toInt32();
+        bPresetId = sal_True;
+    }
+    }
+    else
+    {
+    const preset_maping* p = gPresetMaping;
+    while( p->mpStrPresetId && ((p->mnPresetClass != (sal_Int32)nAPIPresetClass) || !rPreset.equalsAscii( p->mpStrPresetId )) )
+        p++;
+
+    if( p->mpStrPresetId )
+    {
+        nPresetId = p->mnPresetId;
+        bPresetId = sal_True;
+    }
+    }
+
+    return nPresetId;
+}
+
 sal_Int16 AnimationExporter::exportAnimPropertySet( SvStream& rStrm, const Reference< XAnimationNode >& xNode )
 {
     sal_Int16 nNodeType = ::com::sun::star::presentation::EffectNodeType::DEFAULT;
 
     EscherExContainer aAnimPropertySet( rStrm, DFF_msofbtAnimPropertySet );
-    const ::com::sun::star::uno::Any* pAny[ DFF_ANIM_PROPERTY_ID_COUNT ];
-    rtl_zeroMemory( pAny, sizeof( pAny ) );
 
     Reference< XAnimationNode > xMaster;
 
-    const Any aTrue( makeAny( (sal_Bool)sal_True ) );
     Any aMasterRel, aOverride, aRunTimeContext;
 
     // storing user data into pAny, to allow direct access later
-    Sequence< NamedValue > aUserData = xNode->getUserData();
-    if ( aUserData.getLength() )
-    {
-        const NamedValue* p = aUserData.getConstArray();
-        sal_Int32 nLength = aUserData.getLength();
-        while( nLength-- )
-        {	
-            if( p->Name.equalsAsciiL( RTL_CONSTASCII_STRINGPARAM( "node-type" ) ) )
-            {
-                pAny[ DFF_ANIM_NODE_TYPE ] = &(p->Value);
-            }
-            else if ( p->Name.equalsAsciiL( RTL_CONSTASCII_STRINGPARAM( "preset-class" ) ) )
-            {
-                pAny[ DFF_ANIM_PRESET_CLASS ] = &(p->Value);
-            }
-            else if ( p->Name.equalsAsciiL( RTL_CONSTASCII_STRINGPARAM( "preset-id" ) ) )
-            {
-                pAny[ DFF_ANIM_PRESET_ID ] = &(p->Value);
-            }
-            else if ( p->Name.equalsAsciiL( RTL_CONSTASCII_STRINGPARAM( "preset-sub-type" ) ) )
-            {
-                pAny[ DFF_ANIM_PRESET_SUB_TYPE ] = &(p->Value);
-            }
-            else if ( p->Name.equalsAsciiL( RTL_CONSTASCII_STRINGPARAM( "master-element" ) ) )
-            {
-                pAny[ DFF_ANIM_AFTEREFFECT ] = &aTrue;
-                p->Value >>= xMaster;
-            }
-            p++;
-        }
-    }
+    const Sequence< NamedValue > aUserData = xNode->getUserData();
+    const ::com::sun::star::uno::Any* pAny[ DFF_ANIM_PROPERTY_ID_COUNT ];
+    GetUserData( aUserData, pAny, sizeof( pAny ) );
+
+    if( pAny[ DFF_ANIM_AFTEREFFECT ] )
+        ( *pAny[ DFF_ANIM_AFTEREFFECT ] ) >>= xMaster;
 
     // calculate master-rel
     if( xMaster.is() )
@@ -978,7 +1002,7 @@ sal_Int16 AnimationExporter::exportAnimPropertySet( SvStream& rStrm, const Refer
             sal_uInt32 nPPTNodeType = DFF_ANIM_NODE_TYPE_ON_CLICK;
             switch( nNodeType )
             {
-                case ::com::sun::star::presentation::EffectNodeType::ON_CLICK : nPPTNodeType = DFF_ANIM_NODE_TYPE_ON_CLICK;	break;
+                case ::com::sun::star::presentation::EffectNodeType::ON_CLICK : nPPTNodeType = DFF_ANIM_NODE_TYPE_ON_CLICK; break;
                 case ::com::sun::star::presentation::EffectNodeType::WITH_PREVIOUS : nPPTNodeType = DFF_ANIM_NODE_TYPE_WITH_PREVIOUS; break;
                 case ::com::sun::star::presentation::EffectNodeType::AFTER_PREVIOUS : nPPTNodeType = DFF_ANIM_NODE_TYPE_AFTER_PREVIOUS; break;
                 case ::com::sun::star::presentation::EffectNodeType::MAIN_SEQUENCE : nPPTNodeType = DFF_ANIM_NODE_TYPE_MAIN_SEQUENCE; break;
@@ -993,7 +1017,7 @@ sal_Int16 AnimationExporter::exportAnimPropertySet( SvStream& rStrm, const Refer
     sal_uInt32 nAPIPresetClass = EffectPresetClass::CUSTOM;
     sal_uInt32 nPresetClass = DFF_ANIM_PRESS_CLASS_USER_DEFINED;
     sal_Bool bPresetClass, bPresetId, bPresetSubType;
-    bPresetClass = bPresetId = bPresetSubType = sal_False;
+    bPresetId = bPresetClass = bPresetSubType = sal_False;
 
     if ( pAny[ DFF_ANIM_PRESET_CLASS ] )
     {
@@ -1019,30 +1043,7 @@ sal_Int16 AnimationExporter::exportAnimPropertySet( SvStream& rStrm, const Refer
     {
         rtl::OUString sPreset;
         if ( *pAny[ DFF_ANIM_PRESET_ID ] >>= sPreset )
-        {
-            if ( sPreset.match( rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( "ppt_" ) ), 0 ) )
-            {
-                sal_Int32 nLast = sPreset.lastIndexOf( '_' );
-                if ( ( nLast != -1 ) && ( ( nLast + 1 ) < sPreset.getLength() ) )
-                {
-                    rtl::OUString aNumber( sPreset.copy( nLast + 1 ) );
-                    nPresetId = aNumber.toInt32();
-                    bPresetId = sal_True;
-                }
-               }
-            else
-            {
-                const preset_maping* p = gPresetMaping;
-                while( p->mpStrPresetId && ((p->mnPresetClass != (sal_Int32)nAPIPresetClass) || !sPreset.equalsAscii( p->mpStrPresetId )) )
-                    p++;
-
-                if( p->mpStrPresetId )
-                {
-                    nPresetId = p->mnPresetId;
-                    bPresetId = sal_True;
-                }
-            }
-        }
+            nPresetId = GetPresetID( sPreset, nAPIPresetClass, bPresetId );
     }
 
     if ( pAny[ DFF_ANIM_PRESET_SUB_TYPE ] )
@@ -1050,7 +1051,7 @@ sal_Int16 AnimationExporter::exportAnimPropertySet( SvStream& rStrm, const Refer
         rtl::OUString sPresetSubType;
         if ( *pAny[ DFF_ANIM_PRESET_SUB_TYPE ] >>= sPresetSubType )
         {
-            nPresetSubType = ImplTranslatePresetSubType( nPresetClass, nPresetId, sPresetSubType );
+            nPresetSubType = TranslatePresetSubType( nPresetClass, nPresetId, sPresetSubType );
             bPresetSubType = sal_True;
         }
     }
@@ -1089,9 +1090,7 @@ sal_Int16 AnimationExporter::exportAnimPropertySet( SvStream& rStrm, const Refer
         Reference< XAnimateColor > xColor( xNode, UNO_QUERY );
         if( xColor.is() )
         {
-//			sal_uInt32 nColorSpace = xColor->getColorSpace() == AnimationColorSpace::RGB ? 0 : 1;
-//			exportAnimPropertyuInt32( rStrm, DFF_ANIM_COLORSPACE, nColorSpace, TRANSLATE_NONE );
-            
+
             sal_Bool bDirection = !xColor->getDirection();
             exportAnimPropertyuInt32( rStrm, DFF_ANIM_DIRECTION, bDirection, TRANSLATE_NONE );
         }
@@ -1127,11 +1126,11 @@ sal_Int16 AnimationExporter::exportAnimPropertySet( SvStream& rStrm, const Refer
     }
     if ( pAny[ DFF_ANIM_EVENT_FILTER ] )
     {
-        // TODO DFF_ANIM_EVENT_FILTER 
+        // TODO DFF_ANIM_EVENT_FILTER
     }
     if ( pAny[ DFF_ANIM_VOLUME ] )
     {
-        // TODO DFF_ANIM_VOLUME 
+        // TODO DFF_ANIM_VOLUME
     }
     return nNodeType;
 }
@@ -1289,7 +1288,7 @@ void AnimationExporter::exportAnimEvent( SvStream& rStrm, const Reference< XAnim
 
         switch( i )
         {
-            case 0 : 
+            case 0 :
             case 1 :
             {
                 Any aAny;
@@ -1304,7 +1303,6 @@ void AnimationExporter::exportAnimEvent( SvStream& rStrm, const Reference< XAnim
                         Reference< XEnumeration > xE( xEA->createEnumeration(), UNO_QUERY_THROW );
                         if ( xE.is() && xE->hasMoreElements() )
                         {
-//							while( xE->hasMoreElements() )
                             {
                                 Reference< XAnimationNode > xClickNode( xE->nextElement(), UNO_QUERY );
                                 aAny = xClickNode->getBegin();
@@ -1318,7 +1316,7 @@ void AnimationExporter::exportAnimEvent( SvStream& rStrm, const Reference< XAnim
                     else
                     {
                         aAny = xNode->getBegin();
-                        if ( nFlags & 0x10 )	// replace ON_NEXT with IDEFINITE
+                        if ( nFlags & 0x10 )    // replace ON_NEXT with IDEFINITE
                         {
                             if ( ( aAny >>= aEvent ) && ( aEvent.Trigger == EventTrigger::ON_NEXT ) )
                             {
@@ -1413,7 +1411,7 @@ void AnimationExporter::exportAnimEvent( SvStream& rStrm, const Reference< XAnim
     }
 }
 
-Any AnimationExporter::convertAnimateValue( const Any& rSourceValue, const rtl::OUString& rAttributeName ) const
+Any AnimationExporter::convertAnimateValue( const Any& rSourceValue, const rtl::OUString& rAttributeName )
 {
     rtl::OUString aDest;
     if ( rAttributeName.equalsAsciiL( RTL_CONSTASCII_STRINGPARAM( "X" ) )
@@ -1429,7 +1427,7 @@ Any AnimationExporter::convertAnimateValue( const Any& rSourceValue, const rtl::
             aDest += aStr;
         }
     }
-    else if ( rAttributeName.equalsAsciiL( RTL_CONSTASCII_STRINGPARAM( "Rotate" ) )			// "r" or "style.rotation" ?
+    else if ( rAttributeName.equalsAsciiL( RTL_CONSTASCII_STRINGPARAM( "Rotate" ) )         // "r" or "style.rotation" ?
             || rAttributeName.equalsAsciiL( RTL_CONSTASCII_STRINGPARAM( "SkewX" ) )
             || rAttributeName.equalsAsciiL( RTL_CONSTASCII_STRINGPARAM( "Opacity" ) )
             || rAttributeName.equalsAsciiL( RTL_CONSTASCII_STRINGPARAM( "CharHeight" ) )
@@ -1440,7 +1438,7 @@ Any AnimationExporter::convertAnimateValue( const Any& rSourceValue, const rtl::
             aDest += rtl::OUString::valueOf( fNumber );
     }
     else if ( rAttributeName.equalsAsciiL( RTL_CONSTASCII_STRINGPARAM( "Color" ) )
-            || rAttributeName.equalsAsciiL( RTL_CONSTASCII_STRINGPARAM( "FillColor" ) )		// "Fillcolor" or "FillColor" ?
+            || rAttributeName.equalsAsciiL( RTL_CONSTASCII_STRINGPARAM( "FillColor" ) )     // "Fillcolor" or "FillColor" ?
             || rAttributeName.equalsAsciiL( RTL_CONSTASCII_STRINGPARAM( "LineColor" ) )
             || rAttributeName.equalsAsciiL( RTL_CONSTASCII_STRINGPARAM( "CharColor" ) )
         )
@@ -1475,7 +1473,7 @@ Any AnimationExporter::convertAnimateValue( const Any& rSourceValue, const rtl::
         if ( rSourceValue >>= eFillStyle )
         {
             if ( eFillStyle == ::com::sun::star::drawing::FillStyle_NONE )
-                aDest += rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( "none" ) );	// ?
+                aDest += rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( "none" ) );    // ?
             else
                 aDest += rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( "solid" ) );
         }
@@ -1521,7 +1519,7 @@ Any AnimationExporter::convertAnimateValue( const Any& rSourceValue, const rtl::
             if ( eFontSlant == com::sun::star::awt::FontSlant_ITALIC )
                 aDest += rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( "italic" ) );
             else
-                aDest += rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( "normal" ) );	// ?
+                aDest += rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( "normal" ) );  // ?
         }
     }
     else if ( rAttributeName.equalsAsciiL( RTL_CONSTASCII_STRINGPARAM( "Visibility" ) ) )
@@ -1551,8 +1549,8 @@ void AnimationExporter::exportAnimateSet( SvStream& rStrm, const Reference< XAni
         EscherExContainer aAnimateSet( rStrm, DFF_msofbtAnimateSet, 0 );
         {
             EscherExAtom aAnimateSetData( rStrm, DFF_msofbtAnimateSetData );
-            sal_uInt32 nId1 = 1;			// ??
-            sal_uInt32 nId2 = 1;			// ??
+            sal_uInt32 nId1 = 1;            // ??
+            sal_uInt32 nId2 = 1;            // ??
             rStrm << nId1 << nId2;
         }
         Any aConvertedValue( convertAnimateValue( xSet->getTo(), xSet->getAttributeName() ) );
@@ -1562,20 +1560,14 @@ void AnimationExporter::exportAnimateSet( SvStream& rStrm, const Reference< XAni
     }
 }
 
-sal_uInt32 GetValueTypeForAttributeName( const rtl::OUString& rAttributeName )
+sal_uInt32 AnimationExporter::GetValueTypeForAttributeName( const rtl::OUString& rAttributeName )
 {
     sal_uInt32 nValueType = 0;
 
-/*
-    AnimationValueType::STRING == 0;
-    AnimationValueType::NUMBER == 1;
-    AnimationValueType::COLOR  == 2;
-*/
-
     struct Entry
     {
         const sal_Char* pName;
-        sal_uInt8		nType;
+        sal_uInt8       nType;
     };
     static const Entry lcl_attributeMap[] =
     {
@@ -1630,7 +1622,7 @@ void AnimationExporter::exportAnimate( SvStream& rStrm, const Reference< XAnimat
 
         EscherExContainer aContainer( rStrm, DFF_msofbtAnimate, 0 );
         {
-            EscherExAtom	aAnimateData( rStrm, DFF_msofbtAnimateData );
+            EscherExAtom    aAnimateData( rStrm, DFF_msofbtAnimateData );
             sal_uInt32 nBits = 0x38;
             sal_Int16 nTmp = xAnimate->getCalcMode();
             sal_uInt32 nCalcMode = /* (nTmp == AnimationCalcMode::FORMULA) ? 2 : */ (nTmp == AnimationCalcMode::LINEAR) ? 1 : 0;
@@ -1679,7 +1671,7 @@ void AnimationExporter::exportAnimateTarget( SvStream& rStrm, const Reference< X
             if ( xAnimate.is() )
             {
                 if ( xAnimate->getAttributeName().getLength() )
-                    nBits |= 4;		// what is attributeName ?, maybe this is set if a DFF_msofbtAnimateAttributeNames is written
+                    nBits |= 4;     // what is attributeName ?, maybe this is set if a DFF_msofbtAnimateAttributeNames is written
                 sal_Int16 nAdditiveMode = xAnimate->getAdditive();
                 if ( nAdditiveMode != AnimationAdditiveMode::BASE )
                 {
@@ -1709,9 +1701,9 @@ void AnimationExporter::exportAnimateTarget( SvStream& rStrm, const Reference< X
             rtl::OUString aAttributeName( xAnimate->getAttributeName() );
             if ( nForceAttributeNames )
             {
-                switch( nForceAttributeNames )
+                if( nForceAttributeNames == 1 )
                 {
-                    case 1 : aAttributeName = rtl::OUString::createFromAscii( "r" ); break;
+                    aAttributeName = rtl::OUString(RTL_CONSTASCII_USTRINGPARAM( "r" ));
                 }
             }
             sal_Int32 nIndex = 0;
@@ -1737,54 +1729,69 @@ void AnimationExporter::exportAnimateTarget( SvStream& rStrm, const Reference< X
     }
 }
 
-void AnimationExporter::exportAnimateTargetElement( SvStream& rStrm, const Any aAny, const sal_Bool bCreate2b01Atom )
+Reference< XShape > AnimationExporter::getTargetElementShape( const Any& rAny, sal_Int32& rBegin, sal_Int32& rEnd, sal_Bool& rParagraphTarget )
 {
     Reference< XShape > xShape;
-    aAny >>= xShape;
-    sal_uInt32 nRefMode = 0;	// nRefMode == 2 -> Paragraph
-    sal_Int32 begin = -1;
-    sal_Int32 end = -1;
+    rAny >>= xShape;
+
+    rParagraphTarget = sal_False;
 
     if( !xShape.is() )
     {
-        ParagraphTarget aParaTarget;
-        if( aAny >>= aParaTarget )
-            xShape = aParaTarget.Shape;
-        if ( xShape.is() )
+    ParagraphTarget aParaTarget;
+    if( rAny >>= aParaTarget )
+        xShape = aParaTarget.Shape;
+    if ( xShape.is() )
+    {
+        // now calculating the character range for the paragraph
+        sal_Int16 nParagraph = aParaTarget.Paragraph;
+        Reference< XSimpleText > xText( xShape, UNO_QUERY );
+        if ( xText.is() )
         {
-            // now calculating the character range for the paragraph
-            sal_Int16 nParagraph = aParaTarget.Paragraph;
-            Reference< XSimpleText > xText( xShape, UNO_QUERY );
-            if ( xText.is() )
+        rParagraphTarget = sal_True;
+        Reference< XEnumerationAccess > xTextParagraphEnumerationAccess( xText, UNO_QUERY );
+        if ( xTextParagraphEnumerationAccess.is() )
+        {
+            Reference< XEnumeration > xTextParagraphEnumeration( xTextParagraphEnumerationAccess->createEnumeration() );
+            if ( xTextParagraphEnumeration.is() )
             {
-                nRefMode = 2;
-                Reference< XEnumerationAccess > xTextParagraphEnumerationAccess( xText, UNO_QUERY );
-                if ( xTextParagraphEnumerationAccess.is() )
+            sal_Int16 nCurrentParagraph;
+            rBegin = rEnd = nCurrentParagraph = 0;
+            while ( xTextParagraphEnumeration->hasMoreElements() )
+            {
+                Reference< XTextRange > xTextRange( xTextParagraphEnumeration->nextElement(), UNO_QUERY );
+                if ( xTextRange.is() )
                 {
-                    Reference< XEnumeration > xTextParagraphEnumeration( xTextParagraphEnumerationAccess->createEnumeration() );
-                    if ( xTextParagraphEnumeration.is() )
-                    {
-                        sal_Int16 nCurrentParagraph;
-                        begin = end = nCurrentParagraph = 0;
-                        while ( xTextParagraphEnumeration->hasMoreElements() )
-                        {
-                            Reference< XTextRange > xTextRange( xTextParagraphEnumeration->nextElement(), UNO_QUERY );
-                            if ( xTextRange.is() )
-                            {
-                                rtl::OUString aParaText( xTextRange->getString() );
-                                sal_Int32 nLength = aParaText.getLength() + 1;
-                                end += nLength;
-                                if ( nCurrentParagraph == nParagraph )
-                                    break;
-                                nCurrentParagraph++;
-                                begin += nLength;
-                            }
-                        }
-                    }
+                rtl::OUString aParaText( xTextRange->getString() );
+                sal_Int32 nLength = aParaText.getLength() + 1;
+                rEnd += nLength;
+                if ( nCurrentParagraph == nParagraph )
+                    break;
+                nCurrentParagraph++;
+                rBegin += nLength;
                 }
             }
+            }
+        }
         }
     }
+    }
+
+    return xShape;
+}
+
+void AnimationExporter::exportAnimateTargetElement( SvStream& rStrm, const Any aAny, const sal_Bool bCreate2b01Atom )
+{
+    sal_uInt32 nRefMode = 0;	// nRefMode == 2 -> Paragraph
+    sal_Int32 begin = -1;
+    sal_Int32 end = -1;
+    sal_Bool bParagraphTarget;
+
+    Reference< XShape > xShape = getTargetElementShape( aAny, begin, end, bParagraphTarget );
+
+    if( bParagraphTarget )
+        nRefMode = 2;
+
     if ( xShape.is() || bCreate2b01Atom )
     {
         EscherExContainer aAnimateTargetElement( rStrm, DFF_msofbtAnimateTargetElement );
@@ -1792,7 +1799,7 @@ void AnimationExporter::exportAnimateTargetElement( SvStream& rStrm, const Any a
         {
             EscherExAtom aAnimReference( rStrm, DFF_msofbtAnimReference );
 
-            sal_uInt32 nRefType = 1;	// TODO: nRefType == 2 -> Sound;
+            sal_uInt32 nRefType = 1;    // TODO: nRefType == 2 -> Sound;
             sal_uInt32 nRefId = ((EscherSolverContainer&)mrSolverContainer).GetShapeId( xShape );
 
             rStrm << nRefMode
@@ -1804,7 +1811,7 @@ void AnimationExporter::exportAnimateTargetElement( SvStream& rStrm, const Any a
         if ( bCreate2b01Atom )
         {
             EscherExAtom a2b01Atom( rStrm, 0x2b01 );
-            rStrm << (sal_uInt32)1;		// ?
+            rStrm << (sal_uInt32)1;     // ?
         }
     }
 }
@@ -1912,14 +1919,14 @@ void AnimationExporter::exportTransitionFilter( SvStream& rStrm, const Reference
         EscherExContainer aAnimateFilter( rStrm, DFF_msofbtAnimateFilter );
         {
             EscherExAtom aAnimateFilterData( rStrm, DFF_msofbtAnimateFilterData );
-            sal_uInt32 nBits = 3;		// bit 0 -> use AnimAttributeValue
+            sal_uInt32 nBits = 3;       // bit 0 -> use AnimAttributeValue
                                         // bit 1 -> use nTransition
 
             sal_uInt32 nTransition = xFilter->getMode() ? 0 : 1;
             rStrm << nBits
                   << nTransition;
         }
-        const sal_Char* pFilter = transition::find( xFilter->getTransition(), xFilter->getSubtype(), xFilter->getDirection() );
+        const sal_Char* pFilter = FindTransitionName( xFilter->getTransition(), xFilter->getSubtype(), xFilter->getDirection() );
         if ( pFilter )
         {
             const OUString aStr( OUString::createFromAscii( pFilter ) );
@@ -1936,11 +1943,11 @@ void AnimationExporter::exportAnimateMotion( SvStream& rStrm, const Reference< X
     {
         EscherExContainer aAnimateMotion( rStrm, DFF_msofbtAnimateMotion );
         {
-            {	//SJ: Ignored from import filter
-                EscherExAtom aAnimateMotionData( rStrm, DFF_msofbtAnimateMotionData ); 
+            {   //SJ: Ignored from import filter
+                EscherExAtom aAnimateMotionData( rStrm, DFF_msofbtAnimateMotionData );
                 sal_uInt32 nBits = 0x98;
                 sal_uInt32 nOrigin = 0x2;
-                float fByX = 100.0;	// nBits&1
+                float fByX = 100.0; // nBits&1
                 float fByY = 100.0; // nBits&1
                 float fFromX = 0.0; // nBits&2
                 float fFromY = 0.0; // nBits&2
@@ -1948,11 +1955,7 @@ void AnimationExporter::exportAnimateMotion( SvStream& rStrm, const Reference< X
                 float fToY = 100.0; // nBits&4
                 rStrm << nBits << fByX << fByY << fFromX << fFromY << fToX << fToY << nOrigin;
             }
-/*			?
-            {
-                EscherExAtom aF137( rStrm, 0xf137 );
-            }
-*/
+
             OUString aStr;
             if ( xMotion->getPath() >>= aStr )
             {
@@ -2015,7 +2018,7 @@ void AnimationExporter::exportAnimateTransform( SvStream& rStrm, const Reference
 
                 // TODO: ZoomContents:
                 //if( nBits & 8 )
-                //(	fprintf( mpFile, " zoomContents=\"%s\"", nZoomContents ? "true" : "false" );
+                //( fprintf( mpFile, " zoomContents=\"%s\"", nZoomContents ? "true" : "false" );
 
                 rStrm << nBits << fByX << fByY << fFromX << fFromY << fToX << fToY << nZoomContents;
             }
@@ -2065,13 +2068,13 @@ sal_Bool AnimationExporter::getColorAny( const Any& rAny, const sal_Int16 nColor
 
     sal_Int32 nColor = 0;
     Sequence< double > aHSL( 3 );
-    if ( rAny >>= nColor )		// RGB color
+    if ( rAny >>= nColor )      // RGB color
     {
         rA = (sal_uInt8)( nColor >> 24 );
         rB = (sal_uInt8)( nColor >> 8 );
         rC = (sal_uInt8)( nColor );
     }
-    else if ( rAny >>= aHSL )	// HSL
+    else if ( rAny >>= aHSL )   // HSL
     {
         rA = (sal_Int32) ( aHSL[ 0 ] * 255.0 / 360.0 );
         rB = (sal_Int32) ( aHSL[ 1 ] * 255.0 );
@@ -2137,11 +2140,11 @@ void AnimationExporter::exportIterate( SvStream& rStrm, const Reference< XAnimat
     {
         EscherExAtom aAnimIteration( rStrm, DFF_msofbtAnimIteration );
 
-        float		fInterval = 10.0;
-        sal_Int32	nTextUnitEffect = 0;
-        sal_Int32	nU1 = 1;
-        sal_Int32	nU2 = 1;
-        sal_Int32	nU3 = 0xe;
+        float       fInterval = 10.0;
+        sal_Int32   nTextUnitEffect = 0;
+        sal_Int32   nU1 = 1;
+        sal_Int32   nU2 = 1;
+        sal_Int32   nU3 = 0xe;
 
         sal_Int16 nIterateType = xIterate->getIterateType();
         switch( nIterateType )
@@ -2189,3 +2192,4 @@ void AnimationExporter::exportIterate( SvStream& rStrm, const Reference< XAnimat
 
 } // namespace ppt;
 
+/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
